module Physics (..) where

import Data exposing (..)
import Time exposing (..)


-- ToDo: many redundancies, try resolving if time available


updateCollWalls : PowerupAlterations -> Ball -> Ball
updateCollWalls powAlt ball =
  testCollWall borders ball
    |> \collSite ->
        let
          ( x, y ) =
            ball.coord

          inWarpZone =
            y >= 0 - halfHeight / 2
        in
          case collSite of
            Top ->
              bumpedSite Top ball

            Left ->
              if powAlt.warp && inWarpZone then
                warpBall Left ball
              else
                bumpedSite Left ball

            Right ->
              if powAlt.warp && inWarpZone then
                warpBall Right ball
              else
                bumpedSite Right ball

            Bot ->
              bumpedSite Bot ball

            otherwise ->
              ball


testCollWall : Borders -> Object a -> CollisionSite
testCollWall { width, height } object =
  let
    ( x, y ) =
      object.coord

    shape =
      object.shape

    ( xSize, ySize ) =
      case shape of
        Circle z ->
          ( z * 2, z * 2 )

        Rectangle ( z1, z2 ) ->
          ( z1, z2 )
  in
    if x + xSize / 2 >= width / 2 then
      Right
    else if x - xSize / 2 <= (-width / 2) then
      Left
    else if y + ySize / 2 >= height / 2 then
      Top
      --  else if y-ySize/2 <= (-height/2) then Bot
    else
      None


bumpedSite : CollisionSite -> Ball -> Ball
bumpedSite collSite ball =
  let
    ( xv, yv ) =
      ball.velo

    newVelo =
      case collSite of
        Left ->
          if xv < 0 then
            ( -xv, yv )
          else
            ( xv, yv )

        Right ->
          if xv > 0 then
            ( -xv, yv )
          else
            ( xv, yv )

        Top ->
          if yv > 0 then
            ( xv, -yv )
          else
            ( xv, yv )

        Bot ->
          if yv < 0 then
            ( xv, -yv )
          else
            ( xv, yv )

        otherwise ->
          ( xv, yv )
  in
    { ball | velo = newVelo }


warpBall : CollisionSite -> Ball -> Ball
warpBall collSite ball =
  let
    ( x, y ) =
      ball.coord

    newCoord =
      case collSite of
        Left ->
          ( x + gameWidth, y )

        Right ->
          ( x - gameWidth, y )

        otherwise ->
          ( x, y )
  in
    { ball | coord = newCoord }


updateCollPowerup : Powerup -> Player -> Powerup
updateCollPowerup powerup player =
  case powerup.shape of
    Circle radius1 ->
      powerup

    Rectangle ( powWidth, powHeight ) ->
      let
        powX =
          fst powerup.coord

        powY =
          snd powerup.coord

        playX =
          fst player.coord

        playY =
          snd player.coord
      in
        case player.shape of
          Circle radius1 ->
            let
              xVec =
                powX - playX

              yVec =
                powY - playY
            in
              if radius1 ^ 2 > xVec ^ 2 + yVec ^ 2 then
                { powerup | falling = False, velo = ( 0, 0 ) }
              else
                powerup

          Rectangle ( playWidth, playHeight ) ->
            let
              widthOuterRec =
                (max (powX + powWidth / 2) (playX + playWidth / 2))
                  - (min (powX - powWidth / 2) (playX - playWidth / 2))

              heightOuterRec =
                (max (powY + powHeight / 2) (playY + playHeight / 2))
                  - (min (powY - powHeight / 2) (playY - playHeight / 2))

              collision =
                widthOuterRec
                  < (powWidth + playWidth)
                  && heightOuterRec
                  < (powHeight + playHeight)
            in
              if collision then
                { powerup | falling = False, velo = ( 0, 0 ) }
              else
                powerup


updateCollPlayer : Ball -> Player -> Ball
updateCollPlayer ball player =
  case ball.shape of
    Rectangle f ->
      ball

    Circle radius1 ->
      case player.shape of
        Rectangle ( pWidth, pHeight ) ->
          let
            bx =
              fst ball.coord

            by =
              snd ball.coord

            px =
              fst player.coord

            py =
              snd player.coord

            widthOuterRec =
              (max (bx + radius1) (px + pWidth / 2))
                - (min (bx - radius1) (px - pWidth / 2))

            heightOuterRec =
              (max (by + radius1) (py + pHeight / 2))
                - (min (by - radius1) (py - pHeight / 2))

            collision =
              widthOuterRec
                < (radius1 * 2 + pWidth)
                && heightOuterRec
                < (radius1 * 2 + pHeight)
          in
            if collision then
              let
                xv =
                  fst ball.velo

                yv =
                  snd ball.velo

                velocity =
                  xv ^ 2 + yv ^ 2 |> sqrt

                alpha =
                  (px - bx) / pWidth * pi / 3

                sinus =
                  2 * alpha |> sin

                cosinus =
                  2 * alpha |> cos

                newXv =
                  0 * cosinus - velocity * sinus

                newYv =
                  0 * sinus + velocity * cosinus
              in
                { ball | velo = ( newXv, newYv ) }
            else
              ball

        Circle radius2 ->
          let
            radius =
              radius1 + radius2

            xh =
              (fst player.coord) - (fst ball.coord)

            yh =
              (snd player.coord) - (snd ball.coord)

            distance =
              xh ^ 2 + yh ^ 2 |> sqrt

            collision =
              radius >= distance
          in
            if collision then
              let
                x =
                  fst ball.velo

                -- |> Debug.watch "x"
                y =
                  snd ball.velo

                -- |> Debug.watch "y"
                alpha =
                  (atan2 x y) - (atan2 xh yh)

                sinus =
                  2 * alpha |> sin

                cosinus =
                  2 * alpha |> cos

                xv =
                  x * cosinus - y * sinus

                yv =
                  x * sinus + y * cosinus

                --Verschiebe die Kugel außerhalb des Kreises
                newX =
                  fst player.coord - xh * (radius + 1) / distance

                newY =
                  snd player.coord - yh * (radius + 1) / distance

                --    velocity = Debug.watch "Velocity" ( x^2 + y^2 |> sqrt)
              in
                { ball | velo = ( -xv, -yv ), coord = ( newX, newY ) }
            else
              ball


updateCollBrick : Ball -> Brick -> Float -> ( Ball, Brick, Float )
updateCollBrick ball brick distance =
  case ball.shape of
    Rectangle f ->
      ( ball, brick, distance )

    Circle radius1 ->
      case brick.shape of
        Circle r ->
          ( ball, brick, distance )

        Rectangle ( brickWidth, brickHeight ) ->
          let
            ballX =
              fst ball.coord

            ballY =
              snd ball.coord

            ballLastX =
              fst ball.lastCoord

            ballLastY =
              snd ball.lastCoord

            brickX =
              fst brick.coord

            brickY =
              snd brick.coord

            widthOuterRec =
              (max (ballX + radius1) (brickX + brickWidth / 2))
                - (min (ballX - radius1) (brickX - brickWidth / 2))

            heightOuterRec =
              (max (ballY + radius1) (brickY + brickHeight / 2))
                - (min (ballY - radius1) (brickY - brickHeight / 2))

            collision =
              widthOuterRec
                < (radius1 * 2 + brickWidth)
                && heightOuterRec
                < (radius1 * 2 + brickHeight)

            currentDistance =
              (ballLastX - brickX) ^ 2 + (ballLastY - brickY) ^ 2
          in
            if collision then
              if distance < currentDistance then
                ( ball, { brick | armor = brick.armor }, distance )
              else
                --choose site which has the greatest distance to the borders of the brick
                let
                  ballXf =
                    (ballX - ballLastX) / 4 + ballLastX

                  ballYf =
                    (ballY - ballLastY) / 4 + ballLastY

                  right =
                    ( ballXf - (brickX + brickWidth / 2), Right )

                  -- |> Debug.watch "right"
                  left =
                    ( brickX - brickWidth / 2 - ballXf, Left )

                  --  |> Debug.watch "left"
                  top =
                    ( ballYf - (brickY + brickHeight / 2), Top )

                  --  |> Debug.watch "top"
                  bot =
                    ( brickY - brickHeight / 2 - ballYf, Bot )

                  --    |> Debug.watch "bot"
                  ( xv, yv ) =
                    ball.velo

                  newVelo =
                    (if fst right < fst left then
                      left
                     else
                      right
                    )
                      |> (\x ->
                            if fst x < fst top then
                              top
                            else
                              x
                         )
                      |> (\y ->
                            if fst y < fst bot then
                              bot
                            else
                              y
                         )
                      |> snd
                      |> \collSite ->
                          if ball.godmode then
                            ball.velo
                          else
                            case Debug.watch "collSite" collSite of
                              Left ->
                                if xv > 0 then
                                  ( -xv, yv )
                                else
                                  ball.velo

                              Right ->
                                if xv < 0 then
                                  ( -xv, yv )
                                else
                                  ball.velo

                              Top ->
                                if yv < 0 then
                                  ( xv, -yv )
                                else
                                  ball.velo

                              Bot ->
                                if yv > 0 then
                                  ( xv, -yv )
                                else
                                  ball.velo

                              None ->
                                ball.velo
                in
                  ( { ball | velo = newVelo }, { brick | armor = brick.armor - 1 }, currentDistance )
            else
              ( ball, brick, distance )



-----------------------------update Coordinates -------------------------------


updateCoord : Object a -> Time.Time -> Object a
updateCoord object time =
  { object
    | coord = calculateCoord object.coord object.velo time
    , lastCoord = object.coord
  }


calculateCoord : ( Float, Float ) -> ( Float, Float ) -> Float -> ( Float, Float )
calculateCoord ( x, y ) ( xv, yv ) time =
  ( x + xv * time, y + yv * time )


boundCoord : Borders -> Object a -> Object a
boundCoord { width, height } object =
  let
    ( x, y ) =
      object.coord

    newX =
      if x > width / 2 then
        width / 2
      else if x < (-width / 2) then
        (-width / 2)
      else
        x

    newY =
      if y > height / 2 then
        height / 2
      else if y < (-height / 2) then
        (-height / 2)
      else
        y
  in
    { object | coord = ( newX, newY ) }
