module Powerup (..) where

import Time
import Random exposing (..)
import Color exposing (..)
import Graphics.Collage
import Physics exposing (..)
import Data exposing (..)


multiBall =
  { defaultPowerup
    | power = MultiBall
    , colour = backgroundColour
    , textColour = black
    , activeTime = -1
    , name = "&#9679;"
  }


doubleScore =
  { defaultPowerup
    | power = DoubleScore
    , colour = lightGreen
    , textColour = black
    , activeTime = 25
    , name = "2x"
  }


roundPlayer =
  { defaultPowerup
    | power = RoundPlayer
    , colour = lightYellow
    , textColour = black
    , activeTime = 15
    , name = "&#9737;"
  }


fastBall =
  { defaultPowerup
    | power = FastBall
    , colour = lightOrange
    , textColour = purple
    , activeTime = 17
    , name = ">>"
  }


enlarge =
  { defaultPowerup
    | power = Enlarge
    , colour = purple
    , textColour = white
    , activeTime = 20
    , name = "&harr;"
  }


extraLife =
  { defaultPowerup
    | power = ExtraLife
    , colour = backgroundColour
    , textColour = red
    , activeTime = -1
    , name = "&#10084;"
  }


godmode =
  { defaultPowerup
    | power = Godmode
    , colour = backgroundColour
    , textColour = black
    , activeTime = 8
    , name = "&#10030;"
  }


warp =
  { defaultPowerup
    | power = Warp
    , colour = blue
    , textColour = white
    , activeTime = 20
    , name = "O"
  }


hammertime =
  { defaultPowerup
    | power = Shoot
    , colour = black
    , textColour = white
    , activeTime = 4
    , name = "&#9760;"
  }



-----------------------------update Powerups-----------------------------------


updatePowerups : List Powerup -> List Powerup -> Player -> Time.Time -> ( List Powerup, List Powerup )
updatePowerups fallingPowerups activePowerups player time =
  deleteFailedPowerups
    (List.map (updateFallingPowerup time player) fallingPowerups)
    (List.map (updateActivePowerup time) activePowerups)


updateFallingPowerup : Time.Time -> Player -> Powerup -> Powerup
updateFallingPowerup time player powerup =
  updateCoord powerup time
    |> \newPowerup ->
        let
          x =
            newPowerup.coord |> Debug.watch "coord"
        in
          updateCollPowerup (newPowerup) player


updateActivePowerup : Time.Time -> Powerup -> Powerup
updateActivePowerup time powerup =
  { powerup | activeTime = (powerup.activeTime - time) }


deleteFailedPowerups : List Powerup -> List Powerup -> ( List Powerup, List Powerup )
deleteFailedPowerups fallingPowerups activePowerups =
  let
    tempActivePowerups =
      deleteActivePowerups (activePowerups |> Debug.watch "active")
  in
    case fallingPowerups of
      powerup :: restPowerups ->
        if powerup.falling then
          if snd powerup.coord < -halfHeight - borderWidth then
            (deleteFailedPowerups restPowerups tempActivePowerups)
          else
            (deleteFailedPowerups restPowerups tempActivePowerups)
              |> \( x, y ) -> ( powerup :: x, y )
        else
          deleteFailedPowerups restPowerups tempActivePowerups
            |> \( x, y ) -> ( x, powerup :: y )

      [] ->
        ( [], tempActivePowerups )


deleteActivePowerups : List Powerup -> List Powerup
deleteActivePowerups activePowerups =
  case activePowerups of
    x :: xs ->
      if x.activeTime < 0 then
        xs
      else
        x :: (deleteActivePowerups xs)

    [] ->
      []


getRandomPowerup : Brick -> Seed -> ( List Powerup, Seed )
getRandomPowerup brick seed =
  let
    ( num, seed' ) =
      generate (int 0 180) seed

    mPowerup =
      if num >= 0 && num < 4 then
        Just multiBall
      else if num >= 10 && num < 13 then
        Just doubleScore
      else if num >= 20 && num < 22 then
        Just roundPlayer
      else if num >= 30 && num < 32 then
        Just fastBall
      else if num >= 40 && num < 43 then
        Just enlarge
      else if num >= 50 && num < 51 then
        Just extraLife
      else if num >= 60 && num < 62 then
        Just godmode
      else if num >= 70 && num < 72 then
        Just warp
      else if num >= 80 && num < 81 then
        Just hammertime
      else
        Nothing
  in
    case mPowerup of
      Just powerup ->
        ( [ { powerup | coord = brick.coord, velo = ( 0, -fallingVelo ) } ], seed' )

      Nothing ->
        ( [], seed' )


getPowerupAlterations : List Powerup -> PowerupAlterations -> Seed -> ( PowerupAlterations, Seed )
getPowerupAlterations powerupList powAlt seed =
  case powerupList of
    power :: restPower ->
      case power.power of
        MultiBall ->
          let
            ( newBall, seed' ) =
              getRandomBall seed
          in
            getPowerupAlterations
              restPower
              { powAlt | newBalls = newBall :: powAlt.newBalls }
              seed'

        DoubleScore ->
          getPowerupAlterations restPower { powAlt | scoreMult = powAlt.scoreMult + 1 } seed

        RoundPlayer ->
          getPowerupAlterations restPower { powAlt | playerShape = Circle playerRadius } seed

        FastBall ->
          getPowerupAlterations restPower { powAlt | ballVeloMult = powAlt.ballVeloMult + 0.3 } seed

        Enlarge ->
          getPowerupAlterations restPower { powAlt | sizeMult = powAlt.sizeMult + 0.5 } seed

        ExtraLife ->
          getPowerupAlterations restPower { powAlt | extraLife = powAlt.extraLife + 1 } seed

        Godmode ->
          getPowerupAlterations restPower { powAlt | godmode = True } seed

        Warp ->
          getPowerupAlterations restPower { powAlt | warp = True } seed

        Shoot ->
          getPowerupAlterations restPower { powAlt | shoot = True } seed

    [] ->
      ( powAlt, seed )
